import string, BaseHTTPServer
from xml.dom.ext.reader import PyExpat
from xml.dom import ext
reader = PyExpat.Reader()

SOAP_ENV_NS = 'http://schemas.xmlsoap.org/soap/envelope/'
ECHO_NS = 'http://uche.ogbuji.net/ws/eg/echo-soapser'


class SoapHandler(BaseHTTPServer.BaseHTTPRequestHandler):
    def do_POST(self):
        clen = self.headers.getheader('content-length')
        if clen:
            clen = string.atoi(clen)
        else:
            print 'POST ERROR: missing content-length'
            return

        if self.path != '/eg/ws/echo-soapser':
            self.send_error(404)
            return
        input_body = self.rfile.read(clen)
        doc = reader.fromString(input_body)

        #This removes extra white-space from the document, which will
        #Make our task easier
        ext.StripXml(doc)
        
        #getElementsByTagNameNS returns a list of elements that match
        #The namespace and local name.
        #The "[0]" gets the first item in the list
        soap_body = doc.getElementsByTagNameNS(SOAP_ENV_NS, 'Body')[0]

        request_element = soap_body.firstChild
        if request_element.localName == 'echo':
            self._echo(request_element)
        else:
            raise ValueError("Unexpected SOAP method: ('%s', '%s')"%(request_element.namespaceURI, request_element.localName))
        return
        
    def _echo(self, requestElem):
        msg_element = requestElem.getElementsByTagNameNS('', 'msg')[0]
        msg = msg_element.firstChild.data

        #Build the response
        response_body = """\
<?xml version='1.0' encoding='utf-8'?>
<SOAP-ENV:Envelope
  xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
  SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
>
  <SOAP-ENV:Body>
    <e:echoResponse xmlns:e="http://uche.ogbuji.net/ws/eg/echo-soapser">
      <msg><![CDATA[%s]]></msg>
    </e:echoResponse>
  </SOAP-ENV:Body>
</SOAP-ENV:Envelope>
"""%msg

        self._sendResponse(200, 'OK', response_body)
        return

    def _sendResponse(self, code, msg, body):
        self.send_response(200, 'OK')
        self.send_header('Content-type','text/html; charset=utf-8')
        self.send_header("Connection", "close")
        self.send_header("Accept-Ranges", "bytes")
        self.send_header('Content-length', len(body)-1)
        self.end_headers()
        self.wfile.write(body)
        return


thisport = 8888
server_address = ('127.0.0.1', thisport)
httpd = BaseHTTPServer.HTTPServer(server_address, SoapHandler)
print "Listening on port", thisport
httpd.serve_forever()
    
