package com.organizedCampgrounds;

import java.net.*; 
import org.apache.soap.*;  
import org.apache.soap.rpc.*; 

    public class Client { 
      public static void main( String[] args ) throws Exception { 
        /*
         * Define the location where the campground may be found and
         * the identifier for the campground object.
         */
        URL url = new URL( "http://localhost:8080/soap/servlet/rpcrouter" ); 
        String urn = "urn:Campground"; 

        /*
         * Set up the call
         */
        Call call = new Call();
        call.setTargetObjectURI( urn ); 
        call.setMethodName( "computePrice" ); 
        call.setEncodingStyleURI( Constants.NS_URI_SOAP_ENC ); 
        java.util.Vector params = new java.util.Vector(); 
        params.addElement( new Parameter( "dayOfWeek", Integer.class, new Integer(args[0]), null ) ); 
        call.setParams( params ); 

        try { 
          System.out.println("Contacting campground");
          System.out.println(" URL: " + url);
          System.out.println(" URN: " + urn); 

          /*
           * Make the call
           */
          Response response = call.invoke( url, "" ); 

          if( !response.generatedFault()){ 
            Parameter result = response.getReturnValue(); // response was OK 
            int price = ((Integer)(result.getValue())).intValue();
            System.out.println( "Daily price= " + price ); 
          } 
          else { 
            /*
             * Error from the campground object
             */
            Fault f = response.getFault(); 
            System.err.println("Campground error: ");
            System.err.println(" " + f.getFaultCode());
            System.err.println(" " + f.getFaultString()); 
          } 
        } 
        catch( SOAPException e ) { 
            /*
             * Error trying to make the call
             */
		Throwable target = e.getTargetException();
		System.out.println("Call error: " + target.getMessage());
	  	target.printStackTrace();
            System.err.println("SOAPException: ");
            System.err.println(" " + e.getFaultCode());  
            System.err.println(" " + e.getMessage()); 
        } 
      } 
    } 


