package com.organizedCampgrounds;

import java.net.*; 
import org.apache.soap.*;  
import org.apache.soap.rpc.*; 
import org.apache.soap.encoding.*;
import org.apache.soap.encoding.soapenc.*;
import org.apache.soap.util.xml.QName;

    public class Client { 
      public static void main( String[] args ) throws Exception { 
        /*
         * Define the location where the campground may be found and
         * the identifier for the campground object.
         */
        URL url = new URL( "http://localhost:8080/soap/servlet/rpcrouter" ); 
        String urn = "urn:Campground"; 
        int nCampsites=0;
        Response response;
        /*
         * Set up the call
         */
        Call call = new Call();
        call.setTargetObjectURI( urn ); 
        call.setEncodingStyleURI( Constants.NS_URI_SOAP_ENC ); 
        java.util.Vector params = new java.util.Vector(); 

        try { 
          System.out.println("Contacting campground");
          System.out.println(" URL: " + url);
          System.out.println(" URN: " + urn); 

          /*
           * Call computePrice
           */
          call.setMethodName( "computePrice" ); 
          params.addElement( new Parameter( "dayOfWeek", Integer.class, new Integer(args[0]), null ) ); 
          call.setParams( params ); 
          response = call.invoke( url, "" ); 
          if( !response.generatedFault()){ 
            Parameter result = response.getReturnValue(); // response was OK 
            int price = ((Integer)(result.getValue())).intValue();
            System.out.println( "Daily price = " + price ); 
          } 
          else { 
            Fault f = response.getFault(); 
            System.err.println("Campground error: \n " + 
                f.getFaultCode() + "\n " + f.getFaultString()); 
          } 

          /*
           * Call getNumberOfCampsites
           */
          call.setMethodName( "getNumberOfCampsites" ); 
          params.clear();
          call.setParams( params ); 
          response = call.invoke( url, "" ); 
          if( !response.generatedFault()){ 
            Parameter result = response.getReturnValue(); // response was OK 
            nCampsites = ((Integer)(result.getValue())).intValue();
            System.out.println( "Number of campsites = " + nCampsites ); 
          } 
          else { 
            Fault f = response.getFault(); 
            System.err.println("Campground error: \n " + 
                f.getFaultCode() + "\n " + f.getFaultString()); 
          } 

          /*
           * Call getCampsite for each campsite
           */
          call.setMethodName( "getCampsite" ); 
          SOAPMappingRegistry registry = new SOAPMappingRegistry();
          QName qname = new QName( "http://proxy.organizedCampgrounds.com", "Campsite" );

          BeanSerializer serializer = new BeanSerializer();
          registry.mapTypes( Constants.NS_URI_SOAP_ENC, qname, Campsite.class, serializer, serializer );
          call.setSOAPMappingRegistry( registry );

          for(int i=0; i<nCampsites; i++){
            params.clear();
            params.addElement( new Parameter( "campsiteNumber", Integer.class, new Integer(i), null ) ); 
            call.setParams( params ); 
            response = call.invoke( url, "" ); 
            if( !response.generatedFault()){ 
              Parameter result = response.getReturnValue(); // response was OK 
              Campsite campsite = ((Campsite)(result.getValue()));
              System.out.println( "Campsite " + i + ": " + campsite.getDescription() + "\n Size: " +
                  campsite.getSize() + " square feet, Shade(1-10):  " + campsite.getShade()); 
            } 
            else { 
              Fault f = response.getFault(); 
              System.err.println("Campground error: \n " + 
                f.getFaultCode() + "\n " + f.getFaultString()); 
            } 
          }
        } 
        catch( SOAPException e ) { 
            /*
             * Error trying to make the call
             */
		Throwable target = e.getTargetException();
		System.out.println("Call error: " + target.getMessage());
	  	target.printStackTrace();
            System.err.println("SOAPException: ");
            System.err.println(" " + e.getFaultCode());  
            System.err.println(" " + e.getMessage()); 
        } 
      } 
    } 


