package com.organizedCampgrounds;


import java.util.Vector;

import com.ibm.uddi.datatype.binding.*;
import com.ibm.wstk.service.definition.*;
import com.ibm.wstk.service.registry.*;
import com.ibm.uddi.util.*;

/**
 * Provide access to a UDDI registry.
 */
public class UDDI{
    /**
     * Use a UDDI registry to find the end point for a service, given
     * a name of the service and the name of the binding port.
     */
    public static String getServiceEndPoint(String serviceName, String portName){
	try {

	    ServiceRegistryProxy serviceRegistry = new ServiceRegistryProxy();
	    ServiceDefinition[] def = serviceRegistry.findServices(serviceName, true);
	    ServiceImplementation impl = def[0].getServiceImplementation();

          /*
           * Of the potentially multiple binding templates, find the one
           * that matches the specified portName.  First, find the position
           * of the specified port in the portNames array....
           */
	    String[] portNames = impl.getPortNames();
	    int portIndex=0;
          boolean found = false;
	    for (int i = 0; i < portNames.length; i++) {
		if (portNames[i].equals(portName)) {
		  portIndex = i; 
              found = true;
		  break;
		}
	    }
	    if (found == false) 
		return(null);

          /*
           * ... Next, use that index to look up the BindingTemplate in 
           * the BindingTemplate Vector.
           */
	    Vector v = impl.getBindingTemplateList();	
          BindingTemplate template = (BindingTemplate) 
            impl.getBindingTemplateList().elementAt(portIndex);
          return(template.getAccessPoint().getText());

	} catch (ServiceRegistryProxyException ex) {
          System.err.println("Registry lookup error: " + ex.getMessage());    
	} 
	return null;
    }    


}
	
	   

