import java.io.*;
import java.net.*;
import java.util.*;

import com.ibm.network.mail.base.*;

import com.ibm.network.ftp.protocol.*;
import com.ibm.network.ftp.event.*;
import com.ibm.network.ftp.*;
import org.apache.soap.util.net.*;
import org.apache.soap.util.*;
import org.apache.soap.rpc.SOAPContext;
import org.apache.soap.Constants;
import org.apache.soap.transport.*;

public class FTP2HTTPBridge implements Runnable {
  Object waitObject = new Object ();
  long pollDelayMillis;
  final URL httpURL;
  FTPProtocol ftpBean = new com.ibm.network.ftp.protocol.FTPProtocol ();
  private SOAPContext responseSOAPContext;
  String myFtpServer;
  String myFtpLogin;
  String myFtpPassword;
  
	public FTP2HTTPBridge (long pollDelayMillis,
                          String ftpServer, 
						  String ftpLoginName,
                          String password,
                          URL httpURL) 
	{
		/* set up the ftp */
		this.pollDelayMillis = pollDelayMillis;
		myFtpServer = ftpServer;
		myFtpLogin = ftpLoginName;
		myFtpPassword = password;
    
		/* set up the http side */
		this.httpURL = httpURL;
		ftpBean.connectLogin (ftpServer, ftpLoginName, password);
		ftpBean.addRemoteFileListListener (new com.ibm.network.ftp.event.RemoteFileListListener () 
			{
				public void remoteFileListReceived (RemoteFileListEvent remoteEvent)
				{
					// This comes back as a Vector of FileInfo classes.
					Vector files; 
					FileInfo fileInfoName;
					String fileName;
					
					files = remoteEvent.getRemoteFileInfo ();
					System.err.println ("Number of files = " + files.size());
					
					for (Enumeration e = files.elements() ; e.hasMoreElements() ;) 
					{
						//fileInfoName = (FileInfo) e.nextElement();
						fileName = ((FileInfo) e.nextElement()).getName ();
	
						try 
						{
							if (fileName.startsWith ("SOAP_REQUEST_"))
							{
								System.err.println ("Opening file named " + fileName);
		
								System.err.println ("Getting file " + fileName);
								System.err.println ("local directory is " + ftpBean.getLocalDir ());
								
								FTPProtocol myTest = new com.ibm.network.ftp.protocol.FTPProtocol ();
								myTest.connectLogin (myFtpServer, myFtpLogin, myFtpPassword);

								myTest.getFile (fileName);
								myTest = null;
								
								System.err.println ("about to sleep");
								Thread.sleep (3000);
								
								System.err.println ("Waking up");
								
								MimeBodyPart mbp = new MimeBodyPart (new FileInputStream (new File (fileName)));
								MimeMessage msg = new MimeMessage();

								msg.setContent (mbp, "");

								System.err.println ("Calling callHTTP");
								callHTTP (msg);
							}
						}
						catch (Exception myErr)
						{
							myErr.printStackTrace ();
						}
					}
				}
			}				
										   );
	}

  /**
   * Poll for messages forever.
   */
	public void run () 
	{
		while (true) 
		{
			String myStatus;
			myStatus = ftpBean.getStatus ();
			System.err.println ("Status is " + myStatus);
			System.err.println ("FTP2HTTPBridge: Polling for messages ..");
			ftpBean.list (true);

			try 
			{
				Thread.sleep (pollDelayMillis);
			}
			catch (Exception e) 
			{
				e.printStackTrace ();
			}
		}
	}

	public void callHTTP (MimeMessage msg)
	{
		System.err.println ("in callHTTP");
		//String actionURI = msg.getHeader (Constants.HEADER_SOAP_ACTION);
		//System.err.println ("got actionURI of " + actionURI);
		
	    MimeBodyPart mbp = (MimeBodyPart) msg.getContent ();

		byte[] ba = (byte[]) mbp.getContent ();

		/* forward the content to the HTTP listener as an HTTP POST */
								
		Hashtable headers = new Hashtable ();
		//headers.put (Constants.HEADER_SOAP_ACTION, actionURI);
		TransportMessage response;
		SOAPContext ctx;
		String payload = null;
								
		try
		{
			TransportMessage tmsg = new TransportMessage(new String (ba),
														 new SOAPContext(),
														 headers);
			tmsg.save();
									
			System.err.println ("Calling HTTP post ");
									
			response = HTTPUtils.post (this.httpURL, tmsg, 30000, null, 0);
			ctx = response.getSOAPContext();

			System.err.println ("HTTP RESPONSE IS: ");
									
			payload = IOUtils.getStringFromReader (response.getEnvelopeReader());
		
		    MimeBodyPart mbpResponse = new MimeBodyPart ();
		    mbpResponse.setContent (new String (payload), response.getContentType());
			mbpResponse.setEncoding (MimePart.SEVENBIT);
			mbpResponse.setDisposition (MimePart.INLINE);

		    MimeMessage msgResponse = new MimeMessage();
			msgResponse.setContent (mbpResponse, "");

			File soapMessageFile = File.createTempFile ("SOAP_RESPONSE_", "_RC", new File ("."));
			FileOutputStream out = new FileOutputStream (soapMessageFile);
			InputStream in = msgResponse.getInputStream ();
			
			int c;

	        while ((c = in.read()) != -1)
			{
				System.err.write (c);
				out.write(c);
			}

			out.flush ();
			out.close ();

			System.err.println ("About to do put with file " + soapMessageFile.getName ());

			FTPProtocol myTest = new com.ibm.network.ftp.protocol.FTPProtocol ();
			myTest.connectLogin (myFtpServer, myFtpLogin, myFtpPassword);

			myTest.putFile (soapMessageFile.getName ());
		}
		catch (Exception err) 
		{
			err.printStackTrace ();
		}
		System.err.println ("'" + payload + "'");
	}
	public static void main (String[] args) throws Exception 
	{
		if (args.length != 5) 
		{
			System.err.println ("Usage: java " + FTP2HTTPBridge.class.getName () +
			                    " polldelay \\");
			System.err.println ("\t\t ftphost ftplogin ftppasswd httpurl ");
			System.err.println ("  where:");
			System.err.println ("    polldelay    number of millisec to " +
			                    "sleep between polls");
			System.err.println ("    ftphost     hostname of the FTP server");
			System.err.println ("    ftplogin    login ID of FTP account");
			System.err.println ("    ftppasswd   FTP account password");
			System.err.println ("    routerURL   http URL of SOAP router to " +
			                    "bridge to");
			System.exit (1);
		}
		FTP2HTTPBridge sb = new FTP2HTTPBridge (Integer.parseInt (args[0]),
		                                         args[1],
		                                         args[2],
		                                         args[3],
		                                         /* HTTP params */
		                                         new URL (args[4]));
		new Thread (sb).start ();
	}
}
