import java.io.*;
import java.net.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.soap.util.*;
import org.apache.soap.util.xml.*;
import org.apache.soap.Envelope;
import org.apache.soap.SOAPException;
import org.apache.soap.encoding.*;
import org.apache.soap.transport.*;
import org.apache.soap.rpc.*;

import com.ibm.network.ftp.protocol.*;
import com.ibm.network.ftp.event.*;
import com.ibm.network.ftp.*;
import com.ibm.network.mail.base.*;

public class SOAPFTPConnection implements SOAPTransport {
	private Object waitObject = new Object ();
	private long popPollDelay;
	private boolean responseReceived = false;
	private Hashtable responseHeaders = new Hashtable ();
	private BufferedReader responseReader = null;
	private SOAPContext responseSOAPContext;
	FTPProtocol ftpBean = new FTPProtocol ();
	private String prefix;
	String myFtpServer;
	String myFtpLogin;
	String myFtpPassword;

	public SOAPFTPConnection (String ftpServer, 
	                          int popPollDelay,
	                          String ftpLogin, 
							  String ftpPassword)
	{
		System.err.println ("Constructing SOAPFTPConnection");
		System.err.println ("connecting to " + ftpServer);
		System.err.println ("username is " + ftpLogin);
		System.err.println ("password is " + ftpPassword);
		
		myFtpServer = ftpServer;
		myFtpLogin = ftpLogin;
		myFtpPassword = ftpPassword;

		ftpBean.connectLogin (ftpServer, ftpLogin, ftpPassword);
		prefix = "SOAP_REQUEST_" + ftpServer + "_" + ftpLogin + "_" + ftpPassword;
				
		ftpBean.addRemoteFileListListener (new com.ibm.network.ftp.event.RemoteFileListListener () 
			{
				public void remoteFileListReceived (RemoteFileListEvent remoteEvent)
				{
					System.err.println ("In remoteFileListReceived in SOAPFTPCOnnection");
					// This comes back as a Vector of FileInfo classes.
					Vector files; 
					FileInfo fileInfoName;
					String fileName;
					SOAPContext responseSOAPContext;
					
					files = remoteEvent.getRemoteFileInfo ();
					System.err.println ("Number of files = " + files.size());
					
					for (Enumeration e = files.elements() ; e.hasMoreElements() ;) 
					{
						//fileInfoName = (FileInfo) e.nextElement();
						fileName = ((FileInfo) e.nextElement()).getName ();
	
						if (fileName.startsWith ("SOAP_RESPONSE_"))
						{
							String baStr = null;
							
							try 
							{	
								FTPProtocol myTest = new com.ibm.network.ftp.protocol.FTPProtocol ();
								myTest.connectLogin (myFtpServer, myFtpLogin, myFtpPassword);

								myTest.getFile (fileName);
								//myTest.deleteFile (fileName, true);
								myTest = null;
								
								MimeBodyPart mbp = new MimeBodyPart (new FileInputStream (fileName));
								byte[] ba = (byte[]) mbp.getContent ();
							
								baStr = new String (ba);
								
							    responseSOAPContext = new SOAPContext();
							    responseSOAPContext.setRootPart(baStr, "text/xml");			
								
								responseReader = new BufferedReader (new StringReader (baStr));
								
								System.err.println (baStr);
								
								responseReceived = true;
							} 
							catch(Exception ioe) 
							{
							    ioe.printStackTrace();
							}
							
							System.out.println( "Soap Response file Name is " + fileName);
						}
					}
				}										   
			}
										   );
	}

	public void send (URL toAddr, 
					  String actionURI, 
					  Hashtable headers, 
					  Envelope env,
	                  SOAPMappingRegistry smr, 
					  SOAPContext ctx) 
		  throws SOAPException 
	{
		System.err.println ("in send");
		MimeMessage msg = new MimeMessage();

		MimeBodyPart mbp = new MimeBodyPart ();
		StringWriter sw = new StringWriter();

		try 
		{
			env.marshall (sw, smr);
			
			mbp.setContent (sw.toString (), "text/xml");
			mbp.setEncoding (MimePart.SEVENBIT);
			mbp.setDisposition (MimePart.INLINE);
    
			msg.setContent (mbp, "");
			
			File soapMessageFile = File.createTempFile (prefix, "_RC", new File ("."));
			FileOutputStream out = new FileOutputStream (soapMessageFile);
			InputStream in = msg.getInputStream ();
			out.flush ();
			
			int c;

	        while ((c = in.read()) != -1)
			{
				System.err.write (c);
				out.write(c);
			}

			out.flush ();
			out.close ();

			System.err.println ("About to do put with file " + soapMessageFile.getName ());
			ftpBean.putFile (soapMessageFile.getName ());
		}
		catch (Exception e)
		{
			e.printStackTrace ();
		}

		System.err.println ("returning from Send");
	}

	/**
	 * Return a buffered reader to receive back the response to whatever
	 * was sent to whatever. This does a blocking wait until a response
	 * is received .. don't do it lightly. Should take into account
	 * timeouts and such in the future.
	 *
	 * @return a reader to read the results from or null if that's not
	 *         possible.
	 */
	public synchronized BufferedReader receive () 
	{
		System.err.println ("in receive");
		if (responseReader == null) 
		{
			responseReceived = false;
			while (!responseReceived) 
			{
				try 
				{
					ftpBean.list (true);
		            if (responseReceived) 
					{
						System.err.println ("response recieved, break receive loop");
						break;
					}
					Thread.sleep (popPollDelay);
				} 
				catch (Exception e) 
				{
					e.printStackTrace ();
				}
			}
		}
	    return responseReader;
	}

	/**
	 * Return access to headers generated by the protocol.
	 * 
	 * @return a hashtable containing all the headers
	 */
	public synchronized Hashtable getHeaders () 
	{
		System.err.println ("in getHeaders");
		if (responseReader == null)
		{
			receive();
		}
	    return responseHeaders;
	}

	/**
	 * Return the SOAPContext associated with the response.
	 * Not implemented yet!
	 *
	 * @return response SOAPContext
	 */
	public synchronized SOAPContext getResponseSOAPContext () 
	{
		System.err.println ("in getResponseSOAPContext");
		if (responseReader == null)
		{
			receive();
			System.err.println ("receive returned");
		}
		try {
			javax.mail.internet.MimeBodyPart rootPart = responseSOAPContext.getRootPart();
			System.err.println ("rootPart = " + rootPart);
			if (rootPart.isMimeType("text/*")) {
				System.err.println ("ok");
			}							
			else
			{
				System.err.println ("other mime type");
			}
		}
		catch (Exception error)
		{
		}
		return responseSOAPContext;
	}
}
