package wrox.webservices.chapter8;

import java.util.Date;
import java.text.SimpleDateFormat;

/**
 * The FareService implements a simple web service for our
 * fictitious airline. This service exposes a getFare() method
 * that give fare information for flights.
 */
public class FareService {
  
  /**
   * getFare returns fare information for a flight. The fare information
   * is in US Dollars.
   * @param source       Airport code for the starting airport 
   * @param destination  Airport code for the destination airport 
   * @param date         Date of the flight
   */
   public float getFare (String source, String destination, Date date) {
     /* Our airline has only two flights- one from San Jose (SJC) to
      * Taipei, Taiwan (TPE), and the other to Cancun, Mexico (CAN). 
      */
      if (destination.equals ("TPE")) /* Taipei, Taiwan (TPE) */
        return (float) 899.95;
      else /* Cancun, Mexico (CAN) */
        return (float) 349.95;

     /* The starting airport will always be SJC, and the date parameter 
      * is ignored.  A more realistic fare quoting service should probably 
      * be more complicated than this.
      */
   }

   /** Testing code for the service. No matter how trivial the service,
    *  this kind of test code is always useful.
    */
   public static void main (String args[]) {
     FareService fareService = new FareService ();
     SimpleDateFormat dateFormat = new SimpleDateFormat ("yyyy.MM.dd");
     Date date = null;
     try {
       date = dateFormat.parse ("2002.01.01");
     } catch (Exception e) {/* ignore */}

     System.out.println ("The fare from San Jose to Taipei on " + date 
                      + " is " + fareService.getFare ("SJC", "TPE", date));
     System.out.println ("The fare from San Jose to Cancun on " + date 
                      + " is " + fareService.getFare ("SJC", "CAN", date));
   }

}
