/**
 * FilesystemDAO.java
 *
 */

package com.markrichman.filesystem;

import java.sql.*;


/**
 * FilesystemDAO is responsible for performing the
 * authorization & authentication via a JDBC datasource.
 * @author Mark A. Richman
 * @version 1.0
 */
public class FilesystemDAO {
	protected static final java.lang.String DRIVER  = "sun.jdbc.odbc.JdbcOdbcDriver";
	protected static final java.lang.String URL     = "jdbc:odbc:filesystem";
	protected static final java.lang.String USERID  = "mysql";
	protected static final java.lang.String PASSWD  = "mysql";
	protected Connection con;

	/**
	* Validate UserID with Password.
	* @param userid UserID
	* @param passwd Password
	*/
	public boolean isAuthenticated(String username, String password) {
		System.out.println("isAuthenticated(" + username + ", " + password + ")");
        try {
			Statement stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT password FROM " +
				"principal WHERE username = '" + username + "'");
			if (rs.next()) {		
				if(rs.getString("password").equals(password)) {
					return true;
				}
			}	
			return false;
		}
		catch (SQLException e) {
			return false;
		}
	}

	/**
	* Authorize UserID with Web Service URI
	* @param userid UserID
	* @param uri SOAP Web Service Identifier
	*/
	public boolean isAuthorized(String username, String uri) {
		System.out.println("isAuthorized(" + username + ", " + uri + ")");
        try {
			PreparedStatement pstmt = 
				con.prepareStatement("SELECT p.username FROM " +
				"principal_webservice_map pwm, " +
				"principal p, webservice ws " +
				"WHERE pwm.p_id = p.id " +
				"AND pwm.ws_id = ws.id " +
				"AND p.username = ? AND ws.uri = ?");

			pstmt.setString(1,username);
			pstmt.setString(2,uri);

				ResultSet rs = pstmt.executeQuery();
			if (rs.next()) {
				if(rs.getString("username").equals(username)) {
					return true; 
				}
			}
			return false;
		}
		catch (SQLException e) {
			System.err.println("SQLException: " + e.getMessage());
			return false;
		}
	}

	/**
	 * Gets user's home directory
	 */
	protected String getHomedir(String username) throws Exception {

		String homedir = "";

		try {
			connect();
			Statement stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT homedir FROM " +
				"principal WHERE username = '" + username + "'");
			if (rs.next()) {		
				homedir = rs.getString("homedir");
				disconnect();
				return homedir;
			}
			disconnect();
			throw new Exception("Home directory not found for " + username);
		}
		catch (SQLException e) {
			System.err.println(e.getMessage());
			throw new Exception(e.getMessage());
		}
	}

	/**
	* Open a connection to the datasource.
	*/
	protected synchronized void connect() throws SQLException {
    	System.out.println("connect()");
		try {
			Class.forName(DRIVER).newInstance();
			con = DriverManager.getConnection(URL, USERID, PASSWD);
		}
		catch(SQLException e) {
			System.err.println("SQLException: " + e.getMessage());
		}
		catch(Exception e) {
			System.err.println("Exception: " + e.getMessage());
		}
	}

	/**
	* Close the connection to the datasource.
	*/
	protected synchronized void disconnect() throws Exception {
		System.out.println("disconnect()");
        try {
			con.close();
		}
		catch (Exception e) {
		}
	}
}
