/**
 * FilesystemProxy.java
 *
 */

package com.markrichman.filesystem;

import java.io.*;
import java.net.URL;
import java.util.Vector;
import javax.activation.*;
import javax.mail.internet.*;
import org.apache.soap.Constants;
import org.apache.soap.rpc.Call;
import org.apache.soap.Fault;
import org.apache.soap.rpc.Parameter;
import org.apache.soap.rpc.Response;
import org.apache.soap.transport.http.SOAPHTTPConnection;
import org.apache.soap.util.mime.*;

/**
 *
 * @author Mark A. Richman
 * @version 1.0
 */
public class FilesystemProxy {

	protected String url = "http://localhost:8080/soap/servlet/rpcrouter/";
	protected String username = "setMe";
	protected String password = "setMe";
	protected String targetObjectURI = "urn:filesystem";

	public FilesystemProxy() {
	}

	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getTargetObjectURI() {
		return targetObjectURI;
	}

	public void setTargetObjectURI(String uri) {
		this.targetObjectURI = uri;
	}

	/**
	 * Copies a file
	 * @param sourcePath The path of the file on the remote machine
	 * @param destPath The destination path of the file on the remote machine
	 */
	public void copyTo(String sourcePath, String destPath) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("copyTo");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
		call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
		Vector params = new Vector ();
		params.addElement (new Parameter("sourcePath", String.class, sourcePath, null));
        params.addElement (new Parameter("destPath", String.class, destPath, null));
		call.setParams(params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			//Parameter result = resp.getReturnValue();
			//System.out.println ( result.getValue() );
			return;
		}
	}

	/**
	 * Creates a file
	 * @param filePath The path of the file on the remote machine
	 * @param file The local file to upload
	 */
	public boolean createNewFile(String filePath, File file) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("createNewFile");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
		call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
		call.setParams(params);

		System.out.println("Attaching file: " + file.getName());
		attach(file,call);	// MIME Attachment

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			return true;
		}
	}

	public boolean delete(String filePath) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("delete");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
   		call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			Parameter result = resp.getReturnValue ();
			System.out.println ( result.getValue() );
			Boolean b = (Boolean)result.getValue();
			return b.booleanValue();
		}
	}

	public boolean exists(String filePath) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("exists");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
        call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			Parameter result = resp.getReturnValue ();
			System.out.println ( result.getValue() );
			Boolean b = (Boolean)result.getValue();
			return b.booleanValue();
		}
	}

	public void getFile(String filePath, String local) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("getFile");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
        call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
        	// Write the data
            MimeBodyPart mbp;
            DataHandler dh;
            Object o;
            InputStream is;
            FileOutputStream fos = new FileOutputStream(local);
			
            System.err.println("Saving local file: " + local);
            
            try {
                mbp = (MimeBodyPart)resp.getBodyPart(1);
                dh = mbp.getDataHandler();
                is = dh.getInputStream();

                int c;
                while ((c = is.read()) != -1)
                    fos.write(c);
            } 
            catch(Exception e) {
            	System.err.println(e.getMessage());
                throw new Exception(e.getMessage());
            }
        
        
			//Parameter result = resp.getReturnValue ();
			///System.out.println ( result.getValue() );
		    //File f = new File(local);
			//detach(resp,f);
		}
	}

	public long length(String filePath) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("length");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
        call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			Parameter result = resp.getReturnValue ();
			System.out.println ( result.getValue() );
			Long l = (Long)result.getValue();
			return l.longValue();
		}	
	}

	public String getParent(String filePath) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("getParent");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			Parameter result = resp.getReturnValue ();
			System.out.println ( result.getValue() );
			return result.getValue().toString();
		}	
	}

	public boolean isDirectory(String filePath) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("isDirectory");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			Parameter result = resp.getReturnValue ();
			System.out.println ( result.getValue() );
			Boolean b = (Boolean)result.getValue();
			return b.booleanValue();
		}
	}

	public boolean isFile(String filePath) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("isFile");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			Parameter result = resp.getReturnValue ();
			System.out.println ( result.getValue() );
			Boolean b = (Boolean)result.getValue();
			return b.booleanValue();
		}
	}

	public long lastModified(String filePath) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("lastModified");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
   		call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			Parameter result = resp.getReturnValue ();
			System.out.println ( result.getValue() );
			Long l = (Long)result.getValue();
			return l.longValue();
		}	
	}

	public String[] list(String filePath) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("length");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			Parameter result = resp.getReturnValue ();
			System.out.println ( result.getValue() );
			String[] s = (String[])result.getValue();
			return s;
		}	
	}

	public boolean mkdir(String filePath) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("mkdir");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			Parameter result = resp.getReturnValue ();
			System.out.println ( result.getValue() );
			Boolean b = (Boolean)result.getValue();
			return b.booleanValue();
		}
	}

	public boolean mkdirs(String filePath) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("mkdirs");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			Parameter result = resp.getReturnValue ();
			System.out.println ( result.getValue() );
			Boolean b = (Boolean)result.getValue();
			return b.booleanValue();
		}
	}
    
    public boolean renameTo(String filePath, String newName) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("renameTo");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
        params.addElement (new Parameter("newName", String.class, newName, null));
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			Parameter result = resp.getReturnValue ();
			System.out.println ( result.getValue() );
			Boolean b = (Boolean)result.getValue();
			return b.booleanValue();
		}
	}

	public boolean setLastModified(String filePath, long time) throws Exception {
		Call call = new Call ();
		call.setTargetObjectURI(targetObjectURI);
		call.setMethodName("setLastModified");
		SOAPHTTPConnection hc = new SOAPHTTPConnection();
		hc.setUserName(username);
		hc.setPassword(password);
		call.setSOAPTransport(hc);
   		call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
		Vector params = new Vector ();
		params.addElement (new Parameter("filePath", String.class, filePath, null));
        params.addElement (new Parameter("time", Long.class, new Long(time), null));        
		call.setParams (params);

		Response resp = call.invoke (new URL(url), "");

		if (resp.generatedFault ()) {
			Fault fault = resp.getFault ();
			System.out.println ("Ouch, the call failed: ");
			System.out.println ("  Fault Code   = " + fault.getFaultCode ());  
			System.out.println ("  Fault java.lang.String = " + fault.getFaultString ());
			throw new Exception(fault.getFaultString());
		}
		else {
			Parameter result = resp.getReturnValue ();
			System.out.println ( result.getValue() );
			Boolean b = (Boolean)result.getValue();
			return b.booleanValue();
		}
	}

	protected void attach(File file, Call call) {
		try {
			DataSource ds = new ByteArrayDataSource(file,null);
			DataHandler dh = new DataHandler(ds);
			MimeBodyPart part = new MimeBodyPart();
			part.setDataHandler(dh) ;
			call.addBodyPart(part);
		} 
		catch(Exception e) {
			System.err.println(e.getMessage());
		}

	}

	protected void detach(Response resp, File f) {
		// Write the data
        System.err.println("In detach() for file: " + f.getName());
		try {
			MimeBodyPart mbp;
			DataHandler dh;
			Object o;
			InputStream is;
			FileOutputStream fos = new FileOutputStream(f);

			mbp = (MimeBodyPart)resp.getBodyPart(1);
			dh = mbp.getDataHandler();
			is = dh.getInputStream();

			int c;
			while ((c = is.read()) != -1)
				fos.write(c);
		} 
		catch(Exception e) {
			System.err.println(e.getMessage());
		}

	}

}
