Attribute VB_Name = "SMOGHelp"
'
' Microsoft SOAP Message Object Helpers
'

Option Explicit

Private Declare Function GetTimeZoneInformation Lib "kernel32" (lpTimeZoneInformation As TIME_ZONE_INFORMATION) As Long
Private Type SYSTEMTIME
        wYear As Integer
        wMonth As Integer
        wDayOfWeek As Integer
        wDay As Integer
        wHour As Integer
        wMinute As Integer
        wSecond As Integer
        wMilliseconds As Integer
End Type
Private Type TIME_ZONE_INFORMATION
        Bias As Long
        StandardName(32) As Integer
        StandardDate As SYSTEMTIME
        StandardBias As Long
        DaylightName(32) As Integer
        DaylightDate As SYSTEMTIME
        DaylightBias As Long
End Type

Public Const XML_DATE As String = "yyyy-mm-dd"
Public Const XML_DATETIME As String = "yyyy-mm-ddThh:mm:ss"
Private Const DOT As String = "."
Private Const COLON As String = ":"
Public Const AT As String = "@"

Public Const DEFAULT_INT = 0
Public Const DEFAULT_FLOAT = 0#
Public Const DEFAULT_DATE = "1900-01-01"
Public Const DEFAULT_BOOL = 0
Public Const DEFAULT_STRING = vbNullString

Public Function GetCurrentDateTimeTZ() As String
Attribute GetCurrentDateTimeTZ.VB_Description = "Used to get the current time in date.time.tz XML format"
Dim tz As TIME_ZONE_INFORMATION
Dim ret As Long
ret = GetTimeZoneInformation(tz)
GetCurrentDateTimeTZ = Format(Now, XML_DATETIME) & Format(tz.Bias \ 60, "+00;-00") & COLON & Format(Abs(tz.Bias Mod 60), "00")
End Function

' covert "yyyy-mm-ddThh:mm:ss" to valid vb date format
Function MakeXMLDateTimeValid(sDate As String) As Date
    Dim pos%
    On Error GoTo bad_date:
    
    pos = InStr(sDate, "T")
    If pos = 0 Then
        MakeXMLDateTimeValid = CDate(sDate)
        Exit Function
    End If
    
    MakeXMLDateTimeValid = CDate(Mid(sDate, 1, pos - 1) & " " & Mid(sDate, pos + 1, Len(sDate) - pos))
    Exit Function
    
bad_date:
    Err.Raise vbObjectError + 1, "Invalid XML dateTime format"
End Function

' Validate the "yyyy-mm-ddThh:mm:ss:00" portion of the date
'  Raise an error for invalid formats
Function VaidateXMLDateTimeTZ(sDate As String) As String
    Dim pos1%, pos2%
    Dim d$
    
    pos1 = InStr(sDate, "T")
    If pos1 = 0 Then
        If IsDate(sDate) Then
            VaidateXMLDateTimeTZ = sDate
            Exit Function
        Else
            GoTo bad_date
        End If
    End If
    
    pos2 = InStr(pos1, sDate, "-")
    If pos2 = 0 Then pos2 = InStr(pos1, sDate, "+")

    If pos2 = 0 Then
        pos2 = Len(sDate)
    Else
        pos2 = pos2 - 1
    End If
    
    If pos1 > 0 Then
        d = Mid(sDate, 1, pos1 - 1) & " " & Mid(sDate, pos1 + 1, pos2 - pos1)
    End If
    
    If IsDate(d) Then
        VaidateXMLDateTimeTZ = sDate
        Exit Function
    End If
    
bad_date:
    Err.Raise vbObjectError + 1, "Invalid XML dateTime.tz format"
End Function

Public Function addAttribute(ByVal e As MSXML2.IXMLDOMNode, ByVal Name As String, ByVal NamespaceURI, ByVal Value As String) As MSXML2.IXMLDOMNode
Dim a As MSXML2.IXMLDOMNode

On Error GoTo badadda

Set a = e.ownerDocument.createNode(NODE_ATTRIBUTE, Name, NamespaceURI)
If Value <> vbNullString Then a.nodeValue = Value
Set addAttribute = e.Attributes.setNamedItem(a)
Set a = Nothing
Exit Function

badadda:
Set a = Nothing
Set addAttribute = Nothing
End Function

'
' This helper function adds elements to element nodes
'
Public Function addElement(ByVal e As MSXML2.IXMLDOMNode, ByVal Name As String, ByVal NamespaceURI As String, ByVal Value As String) As MSXML2.IXMLDOMNode
Attribute addElement.VB_Description = "Used to add an element to another element (or document)"
Dim a As MSXML2.IXMLDOMNode

On Error GoTo badadde

Set a = e.ownerDocument.createNode(NODE_ELEMENT, Name, NamespaceURI)
If Value <> "" Then a.Text = Value
e.appendChild a
Set addElement = a
Set a = Nothing
Exit Function

badadde:
Set a = Nothing
Set addElement = Nothing
End Function

Public Function FindElement(ByVal objRootNode As MSXML2.IXMLDOMNode, ByVal strName As String, ByVal strNamespaceURI As String) As MSXML2.IXMLDOMNode

  Dim objChildNode As MSXML2.IXMLDOMNode
  
  For Each objChildNode In objRootNode.childNodes
    If objChildNode.nodeType = NODE_ELEMENT Then
      If objChildNode.baseName = strName And objChildNode.NamespaceURI = strNamespaceURI Then
        Set FindElement = objChildNode
        Exit Function
      End If
    End If
  Next
  
  Set FindElement = Nothing

End Function

Public Function FindAttribute(ByVal objRootNode As MSXML2.IXMLDOMNode, ByVal strName As String, ByVal strNamespaceURI As String) As MSXML2.IXMLDOMNode

  Dim objAttrNode As MSXML2.IXMLDOMNode
  
  For Each objAttrNode In objRootNode.Attributes
    If objAttrNode.baseName = strName And objAttrNode.NamespaceURI = strNamespaceURI Then
      Set FindAttribute = objAttrNode
      Exit Function
    End If
  Next
  
  Set FindAttribute = Nothing

End Function

' FindOptionalAttribute Helper function
' find a particular optional attribute or create if it does not exist
Public Function FindOptionalAttribute(ByVal objRootNode As MSXML2.IXMLDOMNode, ByVal Name As String, ByVal NamespaceURI As String, ByVal Value As String) As MSXML2.IXMLDOMNode
Dim objNode As MSXML2.IXMLDOMNode

On Error GoTo createAttribute
    If objRootNode Is Nothing Then
        Set FindOptionalAttribute = Nothing
        Exit Function
    End If
    
    Set objNode = FindAttribute(objRootNode, Name, NamespaceURI)
    If objNode Is Nothing Then
createAttribute:
        Set FindOptionalAttribute = addAttribute(objRootNode, Name, NamespaceURI, Value)
        Exit Function
    End If
    
    Set FindOptionalAttribute = objNode
    Set objNode = Nothing
End Function

' FindOptionalElement Helper function
' find a particular optional element or create if it does not exist
Public Function FindOptionalElement(ByVal objRootNode As MSXML2.IXMLDOMNode, ByVal Name As String, ByVal NamespaceURI As String, ByVal Value As String) As MSXML2.IXMLDOMNode
Dim objNode As MSXML2.IXMLDOMNode

On Error GoTo createElement
    If objRootNode Is Nothing Then
        Set FindOptionalElement = Nothing
        Exit Function
    End If
    
    Set objNode = FindElement(objRootNode, Name, NamespaceURI)
    If objNode Is Nothing Then
createElement:
        Set FindOptionalElement = addElement(objRootNode, Name, NamespaceURI, Value)
        Exit Function
    End If
    
    Set FindOptionalElement = objNode
    Set objNode = Nothing
End Function

' find a particular optional element in correct sequence, or create if it does not exist
Public Function FindOptionalSeqElement(ByVal objRootNode As MSXML2.IXMLDOMNode, ByVal Name As String, ByVal NamespaceURI As String, ByRef NameSequence() As String, ByRef NamespaceSequence() As String, ByVal Value As String) As MSXML2.IXMLDOMNode
Dim objNode As MSXML2.IXMLDOMNode

On Error GoTo createElement
    If objRootNode Is Nothing Then
        Set FindOptionalSeqElement = Nothing
        Exit Function
    End If
    
    Set objNode = FindElement(objRootNode, Name, NamespaceURI)
    If objNode Is Nothing Then
createElement:
        Set FindOptionalSeqElement = addElementInSeq(objRootNode, Name, NamespaceURI, NameSequence, NamespaceSequence, Value)
        Exit Function
    End If
    
    Set FindOptionalSeqElement = objNode
    Set objNode = Nothing
End Function

'
' This helper function adds elements to element nodes in the correct sequence
'
Public Function addElementInSeq(ByVal e As MSXML2.IXMLDOMNode, ByVal Name As String, ByVal NamespaceURI As String, ByRef NameSequence() As String, ByRef NamespaceSequence() As String, ByVal Value As String) As MSXML2.IXMLDOMNode
Dim a As MSXML2.IXMLDOMNode
Dim n As MSXML2.IXMLDOMNode
Dim i As Integer
Dim j As Integer

On Error GoTo badadde

Set a = e.ownerDocument.createNode(NODE_ELEMENT, Name, NamespaceURI)
If Value <> vbNullString Then a.Text = Value
j = UBound(NameSequence)
For i = 0 To j
    If NameSequence(i) = Name And NamespaceSequence(i) = NamespaceURI Then Exit For
Next

i = i + 1
For i = i To j
    Set n = FindElement(e, NameSequence(i), NamespaceSequence(i))
    
    If Not n Is Nothing Then Exit For
Next

If Not n Is Nothing Then
    e.insertBefore a, n
Else
    e.appendChild a
End If

Set addElementInSeq = a
Set a = Nothing
Exit Function

badadde:
Set a = Nothing
Set addElementInSeq = Nothing
End Function
